/*~FILE: DMA_EX.c
  (C) Copyright 1999 by SIGNALWARE Corporation.

  Property of: SIGNALWARE Corporation (sm)
               P.O. Box 50225
               Colorado Springs, CO 80949

  ~PRODUCT: AED-xxx - Analog Expansion Daughterboard
  ~VERSION: 3.2

  4/13/99 V1.0 - initial version
  5/20/99 V1.1 - add address of MAP1 evm memory area
  5/25/99 V1.2 - correct FPGA address
  6/04/99 V1.3 - correct clock period calculation
  6/06/99 V1.4 - remove printf statements
  7/28/99 V2.0 - add size parm to program DMA
  9/15/99 V3.0 - add DMAC_EN parm to program DMA
               - remove bit set that starts FPGA
  9/17/99 V3.1 - change to FRAME sync
  11/5/99 V3.2 - make FRAME sync conditional on DMAC_EN
  03/31/00 V4.0 - change address to 0x01600000
  
  ~!

  ~HEADER:
     Code for DMA functions employing expansion
   memory interface for access to the daughter board.
  ~!
  ~WARNING:
    This code should not generally be modified for any application.
    If this code is changed other than to correct an error, please
    change the name of the module to avoid confusion.
   ~! */
#ifdef PC
#define cregister /* nil */
#endif

#include <string.h>
#include <common.h>
#include <board.h>
#include <dma.h>
#include <timer.h>
#include "dma_ex.h"

/* The address which causes the FPGA to send a value */
#define FPGA_READ_ADDR 0x01600000
/*==================================================================*/

static unsigned int pri_ctl = 0;
static unsigned int sec_ctl = 0;
static unsigned int src_addr = 0;
static unsigned int dst_addr = 0;
static unsigned int trans_ctr = 0;

/*==================================================================*/
void program_DMA_channel(void *dest, int count, int nbuf,
                         unsigned int size, unsigned int dmac_en) {
  /* configure primary control reg -
       Count reload global register A
       Src reload global register - none
       Dst reload global register - D */
  LOAD_FIELD(&pri_ctl, DMA_ADDR_NO_MOD, SRC_DIR, SRC_DIR_SZ);
  LOAD_FIELD(&pri_ctl, DMA_ADDR_INC, DST_DIR, DST_DIR_SZ);
  LOAD_FIELD(&pri_ctl, SEN_EXT_INT7, RSYNC, RSYNC_SZ);
  if (dmac_en==DMAC_EN_RSYNC_STAT)
        SET_BIT(&pri_ctl, FS);
  LOAD_FIELD(&pri_ctl, SEN_NONE, WSYNC, WSYNC_SZ);
  SET_BIT(&pri_ctl, TCINT); /* enable interrupt */
  LOAD_FIELD(&pri_ctl, size, ESIZE, ESIZE_SZ);
  SET_BIT(&pri_ctl, PRI);  /* DMA priority */
  LOAD_FIELD(&pri_ctl, DMA_CNT_RELOADA, CNT_RELOAD, 1);
  SET_BIT(&pri_ctl, EMOD);  /* emulation - halt channel */
  LOAD_FIELD(&pri_ctl, DMA_RELOAD_GARD, DST_RELOAD, DST_RELOAD_SZ);

  /* configure secondary control reg */
  SET_BIT(&sec_ctl, FRAME_IE);  /* interrupt at end of frame */
  SET_BIT(&sec_ctl, RDROP_IE);  /* interrupt on dropping a word */
  SET_BIT(&sec_ctl, RSYNC_CLR);  /* clear sync flag - no init read */
  LOAD_FIELD(&sec_ctl, dmac_en, DMAC_EN, DMAC_EN_SZ);
                                 /* transfer the DMAC pin control */
  /* configure transfer counter */
  LOAD_FIELD(&trans_ctr, count, ELEMENT_COUNT, ELEMENT_COUNT_SZ);
  LOAD_FIELD(&trans_ctr, nbuf, FRAME_COUNT, FRAME_COUNT_SZ);

  /* configure source address */
  src_addr = (unsigned int)(FPGA_READ_ADDR);

  /* configure dest address */
  dst_addr = (unsigned int)(dest);

} /* end init_DMA_channel */
/*==================================================================*/
void test_DMA_channel (void *src) {
  int cpuFreqInMhz;
  unsigned int period_reg, ctrl_reg;
  /* reconfigure source address -
       Change source address
       Use element size increment
       Reload src from global register C */
  src_addr = (unsigned int)(src);
  LOAD_FIELD(&pri_ctl, DMA_ADDR_INC, SRC_DIR, SRC_DIR_SZ);
  LOAD_FIELD(&pri_ctl, DMA_RELOAD_GARC, SRC_RELOAD, SRC_RELOAD_SZ);

  /* substitute the timer for the FPGA synchronization */
  LOAD_FIELD(&pri_ctl, SEN_TINT1, RSYNC, RSYNC_SZ);
  cpuFreqInMhz = cpu_freq();
  if (cpuFreqInMhz == ERROR) {
    cpuFreqInMhz = 133;
  } /* end if */
  period_reg = ((cpuFreqInMhz*11)>>3); /* 5.5 us */
  ctrl_reg = MASK_BIT(GO) | MASK_BIT(HLD) | MASK_BIT(CLKSRC);
  TIMER_INIT(1, ctrl_reg, period_reg, 0);
} /* end test_DMA_channel */
/*==================================================================*/
void start_DMA_channel (unsigned short chan) {

  /* init all DMA registers */
  dma_init(chan, pri_ctl, sec_ctl, src_addr, dst_addr, trans_ctr);
  dma_global_init(0, trans_ctr, 0, 0, 0, 0, 0, src_addr, dst_addr);

  /* start channel in autoinitialization mode */
  DMA_AUTO_START(chan);

} /* end start_DMA_channel */
/*==================================================================*/

